/*
 * Test.java
 *
 * Created on 25. prosinec 2003, 3:10
 */

package saiko.compression;


/**
 *
 * @author  Dusan Saiko
 * 
 * class to describe arithmetic compression algorithm in 
 * simple test 
 */
public class ArithmeticTest {
    
    /**
     * Test the arithmetic compression on given message
     * @param aMessage
     * @throws Exception
     */
    public static void test(String aMessage) 
    {
    	//initiate the text for compression
        byte message[]=aMessage.getBytes();
        
        //output the text which will be compressed
        System.out.println("===================================");
        System.out.println("input  : "+new String(message));

        //get statistics for given input - how many times each character occures
        Statistics[] stats=new SourceStatistics(message).getStats(1,true);

        //get the total size of the data = total number of all characters
        int totalSize=message.length;

        //initialize low to be 0 and high to be 1
        double low=0, high=1;
        
        //compress the data into real number
        for(int i=0; i<message.length; i++) {
            byte nextChar=message[i];				//get next char from message
            
            //get the position of selected char from stats 
            //the range where the char lies in stats
            int char_high_range=0, char_low_range=0;
            int n=0;
            do {
            	char_low_range=char_high_range;
            	char_high_range+=stats[n].getCount();
            } while(stats[n++].getData()[0]!=nextChar);
            //code new high and low numbers
            double range=high-low;
            high= low + range*char_high_range/(double)totalSize;
            low = low + range*char_low_range/(double)totalSize;
        }

        //compute the result - optimalize number which fits into boundaries
        String l=String.valueOf(low);
        String h=String.valueOf(high);
        String r="";
        int i=0;
        while(i<l.length() && i<h.length() && l.charAt(i)==h.charAt(i)) {
        	r+=l.charAt(i);
        	i++;
        }
        if(low==0 && high==1) {
        	r="0.5";
        } else if(i<l.length() && i<h.length() && l.charAt(i)!=h.charAt(i)) {
        	int n1=Integer.parseInt(String.valueOf(l.charAt(i)));
        	int n2=Integer.parseInt(String.valueOf(h.charAt(i)));
        	int middle=(n1+n2)/2;
        	r+=String.valueOf(middle);
        	if(middle==n1)
            	r+="5";
        } else {
        	r+="5";
        }
        double result=new Double(r).doubleValue();
        
        //print out result
        System.out.println("low    : "+low);
        System.out.println("result : "+result);
        System.out.println("high   : "+high);

        //clear the message
        message=new byte[message.length];

        //decode the number into message back again
        for(i=0; i<message.length; i++) {
            int high_range=0;
            int low_range=0;            
            for(int n=0; n<stats.length; n++) {
                low_range=high_range;
                high_range+=stats[n].getCount();
                //according to the known range search the character
                if(result*totalSize>=low_range && result*totalSize<=high_range) {
                    byte data=stats[n].getData()[0];
                    message[i]=data;
                    double range=stats[n].getCount()/(double)totalSize;
                    result=result-low_range/(double)totalSize;
                    result=result/(double)range;
                    break;
                }
            }
        }

        //print out the output of de-compression
        System.out.println("output : "+new String(message));
    }
    
    /**
     * 
     * @param args
     */
    public static void main(String[] args) {
		test("HELLO");
		test("HELLO WORLD");
		test("International");
		test("AAAAAAAAA");
		test("AAAA AAAA");
		test("AAAA AAAA B");
		test("1234567890");
	}
}
