/*
 * ExpandedBitBaseInputStream.java
 *
 * Created on 6.1. 2004, 23:10
 */

package saiko.compression;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;


/**
 * Class to provide decoding of data coded by ExpandedBitBaseOutputStream
 *
 * @author  Dusan Saiko
 */
public class ExpandedBitBaseInputStream extends FilterInputStream {
    
    byte alphabet[];        //current alphabet in use
    byte prev_alphabet[];   //previous used alphabed
    byte buffer[]=null;     //data buffer into which data is expanded
    int  buffer_pos=-1;     //position in data buffer
    
    /** 
     * Constructor of new instance of BPEOutputStream
     *
     * @param aOutput (OutputStream) - output stream to write write the result into
     * @param aPassword (String) - password, minimum 9 characters
     *
     * @exception  IOException if an I/O error occurs and can not be written to the aOutput
    */
    public ExpandedBitBaseInputStream(InputStream aInput) throws IOException
    {
        super(aInput);
        
        //read and compare the header
        byte header[]=new byte[ExpandedBitBaseOutputStream.header.length()];
        in.read(header);
        if(!ExpandedBitBaseOutputStream.header.equals(new String(header)))
            throw new IOException("Invalid data format - not EBB data stream");        
    }
    
   /**
     * Reads the next byte from this compressed output stream. 
     *
     * @exception  IOException  if an I/O error occurs.
     */
    public synchronized int read() throws IOException {
        
        //if buffer is empty or we have just read everything, reload the buffer
        if(buffer==null || buffer_pos>=buffer.length) {
            int length;
            length=in.read();
            // read next, if none then exit
            if(length<0) return length; //end of data
            //read the alphabet
            if(length!=0) {
                //read and decompress the data
                alphabet=new byte[length];
                in.read(alphabet);
                prev_alphabet=alphabet;
            } else {
                //previously alphabet should be used
                alphabet=prev_alphabet;
            }
            
            //read length of the original buffer
            length=0;
            for(int i=0; i<2; i++) {
                length=length*256;
                int v=in.read();
                length+=v;
            }
            buffer = new byte[length];
            
            //length of the  compressed data chunk
            length=0;
            for(int i=0; i<2; i++) {
                length=length*256;
                length+=in.read();
            }
            
            byte data[]=new byte[length];
            in.read(data);
            BigInteger value=new BigInteger(data);

            //expand number
            BigInteger base=new BigInteger(String.valueOf(alphabet.length));
            
            //convert it to BASE radix
            for(int i=0; i<buffer.length; i++) {
                BigInteger current = value.mod(base);
                value=value.divide(base);
                buffer[buffer.length-1-i]=current.byteValue();
            }
            
            //reindex the buffer
            for(int i=0; i<buffer.length; i++) {
                int v=((int)(char)buffer[i])&0xFF;
                buffer[i]=alphabet[v];
            }
            
            buffer_pos=0;
        }
        
        //return next byte from the buffer
        return ((int)(char)buffer[buffer_pos++])&0xFF;
    }

      /**
     * reads the array of data
     * @param b - the buffer into which the data is read.
     * @param off - the start offset of the data.
     * @param len - the maximum number of bytes read. 
     * @return the total number of bytes read into the buffer, 
        or -1 if there is no more data because the end of the stream has been reached. 
     */
    public synchronized int read(byte b[], int off, int len)
    throws IOException
    {
        if ((off | len | (off + len) | (b.length - (off + len))) < 0) {
	    throw new IndexOutOfBoundsException();
	} else if (len == 0) {
	    return 0;
	}

	int n = 0;
	while (n < len) {
	    int n1 = read();
	    if (n1 < 0) break;
            b[off+n]=(byte)n1;
	    n ++;
	}
	return n==0 ? -1 : n;
    } 
} //class
