/*
 * DiatomicInputStream.java
 *
 * Created on 11.1. 2004, 01:10
 */

package saiko.compression;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;


/**
 * Class to provide huffman coding decompression
 *
 * @author  Dusan Saiko
 * @see saiko.compression.HuffmanInputStream
 */
public class HuffmanInputStream extends FilterInputStream {
    
    byte buffer[]=null;     //data buffer into which data is expanded
    int  buffer_pos=-1;     //position in data buffer
    
    /** 
     * Constructor of new instance of BPEOutputStream
     *
     * @param aOutput (OutputStream) - output stream to write write the result into
     * @param aPassword (String) - password, minimum 9 characters
     *
     * @exception  IOException if an I/O error occurs and can not be written to the aOutput
    */
    public HuffmanInputStream(InputStream aInput) throws IOException
    {
        super(aInput);
    }
    
   /**
     * Reads the next byte from this compressed output stream. 
     *
     * @exception  IOException  if an I/O error occurs.
     */
    public synchronized int read() throws IOException {
        
        //if buffer is empty or we have just read everything, reload the buffer
        if(buffer==null || buffer_pos>=buffer.length) {
            
            //read the original data size
            int size1=in.read();
            int size2=in.read();

            // read next, if none then exit
            if(size1<0 || size2<0) return -1; //end of data
            
            byte data[]=new byte[((size1<<8)+size2)+1];

            //read the number of statistics
            int length=in.read()+1;
            
            Statistics stats[]=new Statistics[length];
            for(int i=0; i<stats.length; i++) {
                stats[i]=new Statistics(new byte[]{(byte) in.read()});
                size1=in.read();
                size2=in.read();
                stats[i].count=((size1<<8) + size2)+1;
            }
            
            //create the huffman tree
            HuffmanBinaryTree tree=new HuffmanBinaryTree();
            if(stats.length>0) {
                for(int i=0; i<stats.length; i++) {
                    tree.addElement(stats[i].data, stats[i].count);
                }
                tree.compute();
            }

            length=0;
            int offset=0;
            StringBuffer currentCode=new StringBuffer(1024);
            while(length<data.length) {
                String currentByte=Integer.toBinaryString(in.read());
                while(currentByte.length()<8) currentByte="0"+currentByte;
                currentCode.append(currentByte);
                byte b[]=null;
                while(length<data.length && currentCode.length()>offset && (b=tree.getRepresentation(currentCode.toString().substring(offset)))!=null) {
                    data[length++]=b[0];
                    String code=tree.getCode(b);
                    offset+=code.length();
                }
            }
            buffer=data;
            buffer_pos=0;
        }
        
        //return next byte from the buffer
        return ((int)(char)buffer[buffer_pos++])&0xFF;
    }

    /**
     * reads the array of data
     * @param b - the buffer into which the data is read.
     * @param off - the start offset of the data.
     * @param len - the maximum number of bytes read. 
     * @return the total number of bytes read into the buffer, 
        or -1 if there is no more data because the end of the stream has been reached. 
     */
    public synchronized int read(byte b[], int off, int len)
    throws IOException
    {
        if ((off | len | (off + len) | (b.length - (off + len))) < 0) {
	    throw new IndexOutOfBoundsException();
	} else if (len == 0) {
	    return 0;
	}

	int n = 0;
	while (n < len) {
	    int n1 = read();
	    if (n1 < 0) break;
            b[off+n]=(byte)n1;
	    n ++;
	}
	return n==0 ? -1 : n;
    }   
} //class
