/*
 * PBEOutputStream.java
 *
 * Created on 27.12. 2003, 23:10
 */

package saiko.compression;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidParameterException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;


/**
 * Class to provide Password Based Encryption input stream
 *
 * @author  Dusan Saiko
 * @see javax.crypto.Cipher
 * @see javax.crypto.CipherInputStream
 */
public class PBEInputStream extends FilterInputStream {
    
    /** 
     * Constructor of new instance of BPEOutputStream
     *
     * @param aOutput (OutputStream) - output stream to write write the result into
     * @param aPassword (String) - password, minimum 9 characters
     *
     * @exception  IOException if an I/O error occurs and can not be written to the aOutput
    */
    public PBEInputStream(InputStream aInput, String aPassword) 
    throws IOException, InvalidParameterException, NoSuchAlgorithmException, java.security.spec.InvalidKeySpecException, java.security.InvalidKeyException, javax.crypto.NoSuchPaddingException, java.security.InvalidAlgorithmParameterException
    {
        super(aInput);

        if(aPassword==null || aPassword.length()<9) throw 
            new InvalidParameterException("Password has to be at least 9 chars");

        // Begin by creating a salt of 64 bits (8 byte)
        byte [] salt = aPassword.substring(0,8).getBytes();

        // Create the PBEKeySpec with the given password          
        PBEKeySpec keySpec = new PBEKeySpec(aPassword.toCharArray());

        // Create a SecretKeyFactory for PBEWithSHAAndTwofish
        SecretKeyFactory keyFactory =
            SecretKeyFactory.getInstance("PBEWithMD5AndDES");

        // Create our key
        SecretKey key = keyFactory.generateSecret (keySpec); 

        // Now create a parameter spec for our salt and iterations
        PBEParameterSpec paramSpec = new PBEParameterSpec (salt, PBEOutputStream.ITERATIONS);

        //Create a cipher and initialize it for deecryption
        Cipher cipher = Cipher.getInstance("PBEWithMD5AndDES");
        cipher.init (Cipher.DECRYPT_MODE, key, paramSpec);

        in=new CipherInputStream(aInput, cipher);

        byte header[]=new byte[PBEOutputStream.header.length()];
        in.read(header);
        if(!PBEOutputStream.header.equals(new String(header)))
            throw new IOException("Invalid data format - not PBEInputStream data format or invalid password.");
        
    }
    
    public static void main(String aArg[]) throws Exception {
        InputStream input=new PBEInputStream(new FileInputStream("c:/file.txt.cr"),"PASSWORD1");
        OutputStream output= new FileOutputStream("c:/file.txt.2");
        int i;
        while((i=input.read())>=0) output.write(i);
        input.close();
        output.flush();
        output.close();
    }    
}
