/*
 * RLEOutputStream.java
 *
 * Created on 24.12. 2003, 22:13
 */

package saiko.compression;

import java.io.*;

/**
 * Class to provide simple RLE algorithm compression output stream
 *
 * @author  Dusan Saiko
 * @see saiko.compression.RLEInputStream
 */
public class RLEOutputStream extends FilterOutputStream {
    
    /**
     * the header sign on the RLE format data
     * DSCRLE1 Duan Saiko Compression RLE version 1
     */
    public final static String header="DSCRLE1";
    
    private int currentChar;  //last character read
    private int currentCount; //currentChar count
    
    /** 
     * Constructor of new instance of RLEOutputStream
     *
     * @param aOutput (OutputStream) - output stream to write write the result into
     * @exception  IOException if an I/O error occurs and can not be written to the aOutput
    */
    public RLEOutputStream(OutputStream aOutput) 
    throws IOException
    {
        super(aOutput);
        currentChar = -1; //initialize the current char to some out of range value
        currentCount = 0; 
        out.write(header.getBytes());
    }

    /**
     * Flush the remaining data to the output
     */
    private void flushBuffer() throws IOException {
        if (currentCount > 0) {
            if(currentCount<4 && currentChar!=0xFF) {
                for(int i=0; i<currentCount; i++)
                    out.write(currentChar);
            } else {
                out.write(0xFF);
                out.write(currentCount-1);
                out.write(currentChar);
            }
	    currentCount = 0;
            currentChar=-1;
        }
    }

    /**
     * Writes the specified byte to this compressed output stream. 
     *
     * @param      aValue (int)  the byte to be written.
     * @exception  IOException  if an I/O error occurs.
     */
    public synchronized void write(int aValue) throws IOException {
        if(aValue==currentChar && currentCount<256) {
            currentCount++;
        } else {
            flushBuffer();
            currentChar=aValue;
            currentCount++;
        }
    }

  
    /**
     * Flushes this buffered output stream. This forces any buffered 
     * output bytes to be written out to the underlying output stream. 
     *
     * @exception  IOException  if an I/O error occurs.
     * @see        java.io.FilterOutputStream#out
     */
    public synchronized void flush() throws IOException {
        flushBuffer();
	out.flush();
    }     
}
